__all__ = ("XBOX_API_RELYING_PARTY", "BEDROCK_REALMS_API_URL", "MC_VERSION")

XBOX_API_RELYING_PARTY = "http://xboxlive.com"
BEDROCK_REALMS_API_URL = "https://pocket.realms.minecraft.net/"
MC_VERSION = "1.20.0"  # this can be a few versions behind
